@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2008 Red Hat, Inc.
+ * (C) Copyright 2007 - 2009 Red Hat, Inc.
  * (C) Copyright 2007 - 2008 Novell, Inc.
  */
 
@@ -31,6 +31,7 @@
 #include "nm-utils-private.h"
 
 #include "nm-setting-8021x.h"
+#include "nm-setting-bluetooth.h"
 #include "nm-setting-connection.h"
 #include "nm-setting-ip4-config.h"
 #include "nm-setting-ip6-config.h"
@@ -140,7 +141,7 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 static GHashTable *registered_settings = NULL;
 
-#define DEFAULT_MAP_SIZE 13
+#define DEFAULT_MAP_SIZE 15
 
 static struct SettingInfo {
 	const char *name;
@@ -231,6 +232,11 @@ register_default_settings (void)
 	                      NM_SETTING_CDMA_ERROR,
 	                      1);
 
+	register_one_setting (NM_SETTING_BLUETOOTH_SETTING_NAME,
+			      NM_TYPE_SETTING_BLUETOOTH,
+			      NM_SETTING_BLUETOOTH_ERROR,
+			      1);
+
 	register_one_setting (NM_SETTING_WIRELESS_SECURITY_SETTING_NAME,
 	                      NM_TYPE_SETTING_WIRELESS_SECURITY,
 	                      NM_SETTING_WIRELESS_SECURITY_ERROR,
@@ -266,6 +272,11 @@ register_default_settings (void)
 	                      NM_SETTING_IP4_CONFIG_ERROR,
 	                      6);
 
+	register_one_setting (NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	                      NM_TYPE_SETTING_IP6_CONFIG,
+	                      NM_SETTING_IP6_CONFIG_ERROR,
+	                      6);
+
 	/* Be sure to update DEFAULT_MAP_SIZE if you add another setting!! */
 }
 
@@ -545,30 +556,6 @@ nm_connection_compare (NMConnection *a,
 	return info.failed ? FALSE : TRUE;
 }
 
-typedef struct {
-	gboolean success;
-	GSList *all_settings;
-	GError **error;
-} VerifySettingsInfo;
-
-static void
-verify_one_setting (gpointer data, gpointer user_data)
-{
-	NMSetting *setting = NM_SETTING (data);
-	VerifySettingsInfo *info = (VerifySettingsInfo *) user_data;
-
-	if (info->success)
-		info->success = nm_setting_verify (setting, info->all_settings, info->error);
-}
-
-static void
-hash_values_to_slist (gpointer key, gpointer value, gpointer user_data)
-{
-	GSList **list = (GSList **) user_data;
-
-	*list = g_slist_prepend (*list, value);
-}
-
 /**
  * nm_connection_verify:
  * @connection: the #NMConnection to verify
@@ -591,7 +578,10 @@ nm_connection_verify (NMConnection *connection, GError **error)
 {
 	NMConnectionPrivate *priv;
 	NMSetting *s_con;
-	VerifySettingsInfo info;
+	GHashTableIter iter;
+	gpointer value;
+	GSList *all_settings = NULL;
+	gboolean success = TRUE;
 
 	if (error)
 		g_return_val_if_fail (*error == NULL, FALSE);
@@ -616,15 +606,18 @@ nm_connection_verify (NMConnection *connection, GError **error)
 		return FALSE;
 	}
 
-	/* Now, run the verify function of each setting */
-	memset (&info, 0, sizeof (info));
-	info.success = TRUE;
-	info.error = error;
-	g_hash_table_foreach (priv->settings, hash_values_to_slist, &info.all_settings);
+	/* Build up the list of settings */
+	g_hash_table_iter_init (&iter, priv->settings);
+	while (g_hash_table_iter_next (&iter, NULL, &value))
+		all_settings = g_slist_append (all_settings, value);
 
-	g_slist_foreach (info.all_settings, verify_one_setting, &info);
-	g_slist_free (info.all_settings);
-	return info.success;
+	/* Now, run the verify function of each setting */
+	g_hash_table_iter_init (&iter, priv->settings);
+	while (g_hash_table_iter_next (&iter, NULL, &value) && success)
+		success = nm_setting_verify (NM_SETTING (value), all_settings, error);
+
+	g_slist_free (all_settings);
+	return success;
 }
 
 /**
@@ -1012,7 +1005,7 @@ nm_connection_new (void)
 /**
  * nm_connection_new_from_hash:
  * @hash: the #GHashTable describing the connection
- * @error: location of a #GError to return on failure
+ * @error: on unsuccessful return, an error
  *
  * Creates a new #NMConnection from a hash table describing the connection.  See
  * nm_connection_to_hash() for a description of the expected hash table.
@@ -1025,15 +1018,12 @@ NMConnection *
 nm_connection_new_from_hash (GHashTable *hash, GError **error)
 {
 	NMConnection *connection;
-	NMConnectionPrivate *priv;
 
 	g_return_val_if_fail (hash != NULL, NULL);
 
 	connection = nm_connection_new ();
 	g_hash_table_foreach (hash, parse_one_setting, connection);
 
-	priv = NM_CONNECTION_GET_PRIVATE (connection);
-
 	if (!nm_connection_verify (connection, error)) {
 		g_object_unref (connection);
 		return NULL;
@@ -1162,7 +1152,7 @@ nm_connection_class_init (NMConnectionClass *klass)
 						    NM_CONNECTION_SCOPE_UNKNOWN,
 						    NM_CONNECTION_SCOPE_USER,
 						    NM_CONNECTION_SCOPE_UNKNOWN,
-						    G_PARAM_READWRITE));
+						    G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	/**
 	 * NMConnection:path:
@@ -1176,7 +1166,7 @@ nm_connection_class_init (NMConnectionClass *klass)
 						  "Path",
 						  "Path",
 						  NULL,
-						  G_PARAM_READWRITE));
+						  G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	/* Signals */
 
